#include "global.h"
#include "mainwindow.h"
#include "playlist.h"
#include "messenger.h"
#include "aboutdialog.h"
#include "blmfile.h"

#include <qapplication.h>
#include <qmessagebox.h>
#include <qevent.h>
#include <qlistview.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qworkspace.h>

#include "connectionwizard.h"
#include "dummyconnection.h"
#include "consoleconnection.h"
#include "telnetmotherwidget.h"
#include "telnetchildwidget.h"
#include "sbtpmotherwidget.h"
#include "sbtpchildwidget.h"
#include "blinksimwidget.h"
#include "udpfixconnection.h"
#include "udpdynconnection.h"

/* 
 *  Constructs a MainWindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
MainWindow::MainWindow( QWidget* parent,  const char* name, WFlags fl )
    : MainWindowBase( parent, name, fl )
{
//    this->showMaximized();

    // The GLOBAL Messenger THIS MUST BE THE FIRST GLOBAL!
    g_messenger = new Messenger();
    m_LogEntryLast = 0;
    // Connect Log-Message to Loglist
    connect(g_messenger, SIGNAL(signalLogEntry(LogEntry&)), 
            this, SLOT(slotAddLogEntry(LogEntry&)));

    // Send Start-LogMessages
    QString message;
    QDateTime time;

    message = "Sytem starting...";
    time = QDateTime::currentDateTime ();
    LogEntry entry(time,
                    LogEntry::StatusINFO,
                    message);
    g_messenger->slotLogEntry(entry);
    
    // The GLOBAL Playlist-handler
    message = "Initializing Playlist...";
    time = QDateTime::currentDateTime ();
    entry = LogEntry(time,
                    LogEntry::StatusINFO,
                    message);
    g_messenger->slotLogEntry(entry);
    g_playlist = new Playlist(PlayListView, 5);
    g_playlist->update();

    QString filename = g_playlist->getNext();
    BlmMovie* movie = new BlmMovie(filename);
    movie->loadFile(filename);

    // Start a dummy-connection (Frame-Counter)
    DummyConnection* dummy = new DummyConnection(Workspace, "X", 
            WDestructiveClose);
    dummy->show();

    // Connect BlinkSim
    g_messenger->slotLogEntry(LogEntry::StatusINFO,
                    "Starting Blinkenlights simulation widget");
        
    connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
             BlinkSim, SLOT(slotShowFrame(BlmFrame&)));
    
    BlinkSim->setHide(false);

}

/*  
 *  Destroys the object and frees any allocated resources
 */
MainWindow::~MainWindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 * public slot
 */
void MainWindow::slotAddLogEntry(LogEntry& entry)
{
    m_LogEntryLast++;
    QString number;
    number = QString( "%1" ).arg( m_LogEntryLast, 8, 10 );
    QString mydate=entry.time.toString("yyyy-MM-dd - hh:mm:ss.zzz");
    QString status;
    switch (entry.status) {
        case LogEntry::StatusDEBUG:
            status = "DEBUG";
            break;
        case LogEntry::StatusINFO:
            status = "INFO";
            break;
        case LogEntry::StatusWARNING:
            status = "WARNING";
            break;
        case LogEntry::StatusERROR:
            status = "ERROR";
            break;
    }
    new QListViewItem(MessageList, number, mydate, status, entry.message);

}

/*
 * public slot
 */
void MainWindow::fileNew()
{
    ConnectionWizard wizard(this);
    if(wizard.exec() != QDialog::Accepted) {
        return;
    }
    if(wizard.radioDummy->isChecked()) {
        DummyConnection* dummy = new DummyConnection(Workspace, "X", 
                WDestructiveClose);
        dummy->show();
    }
    if(wizard.radioDebug->isChecked()) {
        ConsoleConnection* console = new ConsoleConnection(
                wizard.ConsoleNotifyBox->isChecked(),
                wizard.ConsoleShowBox->isChecked(),
                Workspace, "X", WDestructiveClose);
        console->show();
    }
    if(wizard.radioTelnet->isChecked()) {
        TelnetMotherWidget* telnet = new TelnetMotherWidget(
                wizard.telnetPort->value(),
                wizard.telnetConnections->value(),
                wizard.telnetFullRepaintBox->isChecked(),
                Workspace, "X", WDestructiveClose);
        connect(telnet, SIGNAL(signalNeedChild(int,bool,bool,
                        TelnetMotherWidget*)),
                this, SLOT(slotCreateTelnetChild(int,bool,bool,
                        TelnetMotherWidget*)));
        telnet->show();
    }
    if(wizard.radioFileserver->isChecked()) {
        SbtpMotherWidget* sbtp = new SbtpMotherWidget(
                wizard.fileserverPort->value(),
                wizard.fileserverConnections->value(),
                Workspace, "X", WDestructiveClose);
        connect(sbtp, SIGNAL(signalNeedChild(int,bool,
                        SbtpMotherWidget*)),
                this, SLOT(slotCreateSbtpChild(int,bool,
                        SbtpMotherWidget*)));
        sbtp->show();
    }
    if(wizard.radioUdpFix->isChecked()) {
        UdpFixConnection* udpfix = new UdpFixConnection(
                wizard.UdpFixHost->text(),
                wizard.UdpFixPort->value(),
                Workspace, "X", WDestructiveClose);
        udpfix->show();
    }
    if(wizard.radioUdpDynamic->isChecked()) {
        UdpDynConnection* udpdyn = new UdpDynConnection(
                wizard.UdpDynamicPort->value(),
                wizard.UdpDynamicConnections->value(),
                Workspace, "X", WDestructiveClose);
        udpdyn->show();
    }
}

/*
 * public slot
 */
void MainWindow::slotCreateTelnetChild(int socket, bool repaint, bool full,
        TelnetMotherWidget* mother) {
    TelnetChildWidget* child = new TelnetChildWidget(socket, repaint, full,
            mother,
            Workspace, "X", WDestructiveClose);
    child->show();
}

/*
 * public slot
 */
void MainWindow::slotCreateSbtpChild(int socket, bool full,
        SbtpMotherWidget* mother) {
    SbtpChildWidget* child = new SbtpChildWidget(socket, full, mother,
            Workspace, "X", WDestructiveClose);
    child->show();
}

/*
 * public slot
 */
void MainWindow::filePrint()
{
    qWarning( "MainWindow::filePrint() not yet implemented!" );
}

/*
 * public slot
 */
void MainWindow::fileExit()
{
    int ret = QMessageBox::warning(this, "Close Condor?",
            "Closing Condor will close all connections!\n"
            "\n"
            "Are you sure you want to close?",
            "Yes",
            "No",
           QString::null,
           1, 1);
    if(ret == 0) {
        qApp->quit();
    }

}

/*
 * public slot
 */
void MainWindow::closeEvent(QCloseEvent* event)
{
    event->ignore();
    //qApp->processEvents();
    this->fileExit();
}

/*
 * public slot
 */
void MainWindow::playlistAdd()
{
    qWarning( "MainWindow::playlistAdd() not yet implemented!" );
}

/*
 * public slot
 */
void MainWindow::helpContents()
{
    qWarning( "MainWindow::helpContents() not yet implemented!" );
}

/*
 * public slot
 */
void MainWindow::helpAbout()
{
    AboutDialog* dialog = new AboutDialog(this);
    dialog->show();
}

/*
 * public slot
 */
void MainWindow::windowPreview(bool show)
{
    if(show) {
        BlinkSim->setHide(false);
        BlinkSimFrame->show();
    } else {
        BlinkSim->setHide(true);
        BlinkSimFrame->hide();
    }
}

/*
 * public slot
 */
void MainWindow::windowStatus(bool)
{
    qWarning( "MainWindow::windowStatus(bool) not yet implemented!" );
}

/*
 * public slot
 */
void MainWindow::fileLockScreen()
{
    qWarning( "MainWindow::fileLockScreen() not yet implemented!" );
}

/*
 * public slot
 */
void MainWindow::playlistSkip()
{
    g_playlist->skip();
}


