#include <qlabel.h>
#include <qevent.h>
#include <qsocketdevice.h>
#include <qsocketnotifier.h>
#include <qspinbox.h>

#include "udpdynconnection.h"
#include "global.h"
#include "messenger.h"

/* 
 *  Constructs a UdpDynConnection which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
UdpDynConnection::UdpDynConnection(int port, int maxconn,
        QWidget* parent,  const char* name, WFlags fl )
    : udpdynconnectionbase( parent, name, fl )
{
    m_count = 0;
    m_port = port;
    MaxConnectionsBox->setValue(maxconn);

    m_conn = 0;

    m_dict = new MyDict();
    m_dict->setAutoDelete(true);

    g_messenger->slotLogEntry(LogEntry::StatusINFO, 
            QString("Starting new UdpDyn-Connection Port %1")
            .arg(m_port));

    connect(g_messenger, SIGNAL(signalBlmFrame(BlmFrame&)),
            this, SLOT(slotShowFrame()));

    m_socket = new QSocketDevice(QSocketDevice::Datagram);
    if(!m_socket->bind(QHostAddress(), m_port)) {
        g_messenger->slotLogEntry(LogEntry::StatusERROR, 
            QString("Bind to port failed"));
        delete m_socket;
        close();
        return;
    }

    m_notifier = new QSocketNotifier(m_socket->socket(),QSocketNotifier::Read); 
    connect(m_notifier, SIGNAL(activated(int)),
            this, SLOT(slotSocketEvent(int)));
    m_closeignore = true;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
UdpDynConnection::~UdpDynConnection()
{
    delete m_dict;
    delete m_notifier;
    m_socket->close();
    delete m_socket;
}

/*
 * public slot
 */
void UdpDynConnection::slotShowFrame()
{
    m_count++;
    QString count = QString("Frames: %1").arg( m_count, 8, 10 );
    FrameCountLabel->setText(count);
}

/*
 * public slot
 */
void UdpDynConnection::slotClose()
{
    g_messenger->slotLogEntry(LogEntry::StatusINFO, "Closing UdpDyn-Connection");
    m_closeignore = false;
    close();
}

void UdpDynConnection::closeEvent(QCloseEvent* event)
{
    if(m_closeignore) {
        this->slotClose();
        if(m_closeignore) {
            event->ignore();
        } else {
            event->accept();
        }
    } else {
        event->accept();
    }
}


/*
 * public slot
 */
void UdpDynConnection::slotSocketEvent(int)
{
    int bytecount = m_socket->bytesAvailable();
    char Buffer[bytecount];

    m_socket->readBlock(Buffer, bytecount);
    if(bytecount < 8) {
        qDebug("Too few bytes %d (should be a minimum of 8!)",
                bytecount);
        return;
    }
    // Check Magic
    if(Buffer[0] != (char)0xDE ||
            Buffer[1] != (char)0xAA ||
            Buffer[2] != (char)0xBF ||
            Buffer[3] != (char)0xCD) {
        qDebug("Wrong magic....");
        return;
    }
    QHostAddress peer_host = m_socket->peerAddress();
    int peer_port = m_socket->peerPort();
    QString hostdict = QString("%1:%2")
        .arg(peer_host.toString())
        .arg(peer_port);

    if(bytecount >= 11) {
        if(Buffer[4] == 'R' &&
                Buffer[5] == 'E' &&
                Buffer[6] == 'F' &&
                Buffer[7] == 'R' &&
                Buffer[8] == 'E' &&
                Buffer[9] == 'S' &&
                Buffer[10] == 'H') {
            UdpDynDict* client;
            client = m_dict->find(hostdict);
            if(client == 0) { 
                if(m_conn < MaxConnectionsBox->value()) {
                    client = new UdpDynDict(peer_host, peer_port, m_socket);
                    connect(client, SIGNAL(signalTimeOut(QHostAddress, int)),
                            this, SLOT(slotTimeOut(QHostAddress, int)));
                    m_dict->insert(hostdict, client);
                    m_conn++;
                    QString childtext = QString("Count: %1")
                        .arg( m_conn, 4, 10 );
                    ChildCountLabel->setText(childtext);
                } else {
                    g_messenger->slotLogEntry(LogEntry::StatusWARNING, 
                            QString("UdpDyn on Port %1 FULL!")
                            .arg(m_port));
                }
            } else {
                client->slotRefresh();
            }
            return;
        }  
    }

    if(bytecount >= 9) {
        if(Buffer[4] == 'C' &&
                Buffer[5] == 'L' &&
                Buffer[6] == 'O' &&
                Buffer[7] == 'S' &&
                Buffer[8] == 'E') {
            UdpDynDict* client;
            client = m_dict->find(hostdict);
            if(client != 0) { 
                m_dict->remove(hostdict);
                m_conn--;
                QString childtext = QString("Count: %1").arg( m_conn, 4, 10 );
                ChildCountLabel->setText(childtext);
            }
            return;
        }  
    }

    qDebug("NOT accepting Paket");
}

/*
 * public slot
 */
void UdpDynConnection::slotTimeOut(QHostAddress hostaddress, int port) {

    
    QString hostdict = QString("%1:%2")
        .arg(hostaddress.toString())
        .arg(port);
    UdpDynDict* client;
    client = m_dict->find(hostdict);
    if(client == 0) { 
        qDebug("Unknown client...");
    } else {
        m_dict->remove(hostdict);
        m_conn--;
        QString childtext = QString("Count: %1").arg( m_conn, 4, 10 );
        ChildCountLabel->setText(childtext);
    }
}

