#include <qapplication.h>
#include <qmessagebox.h>
#include <qsocketdevice.h>
#include <qsocketnotifier.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qhostaddress.h>
#include <qvaluelist.h>
#include <qdns.h>

#include "blmfile.h"
#include "visionudpmainwindow.h"
#include "visionudpsettings.h"
#include "blinksimwidget.h"

/* 
 *  Constructs a VisionUDP which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
VisionUDP::VisionUDP( QWidget* parent,  const char* name, WFlags fl )
    : VisionUdpMainBase( parent, name, fl )
{
    m_settings = new VisionUDPSettings();
    m_socket = 0;
    m_notifier = 0;
    m_dns = 0;
    m_count = 0;

    blinkSimWidget1->setHide(false);
    
    m_timer = new QTimer(this);

    connect(m_timer, SIGNAL(timeout()),
            this, SLOT(slotRefresh()));
}

/*  
 *  Destroys the object and frees any allocated resources
 */
VisionUDP::~VisionUDP()
{
    delete m_timer;
    delete m_settings;
}

void VisionUDP::closeEvent(QCloseEvent* event)
{
    event->ignore();
    slotExitAction();
}

/*
 * public slot
 */
void VisionUDP::slotExitAction()
{
    int ret = QMessageBox::warning(this, "Close VisionUDP?",
            "Closing VisionUDP will close the connection!\n"
            "\n"
            "Are you sure you want to close?",
            "Yes",
            "No",
           QString::null,
           1, 1);
    if(ret == 0) {
        char Buffer[9];
        // Magic
        Buffer[0] = (char)0xDE;
        Buffer[1] = (char)0xAA;
        Buffer[2] = (char)0xBF;
        Buffer[3] = (char)0xCD;
        Buffer[4] = 'C';
        Buffer[5] = 'L';
        Buffer[6] = 'O';
        Buffer[7] = 'S';
        Buffer[8] = 'E';

        if(!m_socket->writeBlock(Buffer, 9, m_hostaddress, 
                    m_settings->HostPort->value()) == -1) {
            qDebug("VisionUDP: Could not write bytes");
        }
        qApp->processEvents();
        qApp->quit();
    }
}

/*
 * public slot
 */
void VisionUDP::slotConfigureAction()
{
    m_settings->exec();

    if(m_notifier != 0) {
        delete m_notifier;
    }
    if(m_socket != 0) {
        delete m_socket;
    }

    if(m_dns != 0) {
        delete m_dns;
    }

    m_timer->stop();

    m_socket = new QSocketDevice(QSocketDevice::Datagram);
    if(!m_socket->bind(QHostAddress(), m_settings->ClientPort->value())) {
        qFatal("Bind failed!");
    } else {
        ClientLabel->setText(QString("Client-Port: %1")
                .arg(m_settings->ClientPort->value()));
    }

    m_notifier = new QSocketNotifier(m_socket->socket(),QSocketNotifier::Read);

    connect(m_notifier, SIGNAL(activated(int)),
            this, SLOT(slotSocketEvent(int)));

    m_dns = new QDns(m_settings->HostName->text());
    connect(m_dns, SIGNAL(resultsReady()), 
            this, SLOT(slotDnsReady()));


}


/*
 * public slot
 */
void VisionUDP::slotSocketEvent(int)
{
    int bytecount = m_socket->bytesAvailable();
    char Buffer[bytecount];

    m_socket->readBlock(Buffer, bytecount);
    if(bytecount < 156) {
        qDebug("Too few bytes %d (should be 156)",
                bytecount);
        return;
    }
    // Check Magic
    if(Buffer[0] != (char)0xDE ||
            Buffer[1] != (char)0xAD ||
            Buffer[2] != (char)0xBE ||
            Buffer[3] != (char)0xEF) {
        qDebug("Wrong magic....");
        return;
    }

    if(Buffer[8] != (char)0x00 ||
            Buffer[9] != (char)BLMWIDTH ||
            Buffer[10] != (char)0x00 ||
            Buffer[11] != (char)BLMHEIGHT) {
        qDebug("Wrong size....");
        return;
    }

    BlmFrame frame;
    char* bufpoint = &Buffer[12];
    for(int height = 0; height < BLMHEIGHT; height++) {
        for(int width = 0; width < BLMWIDTH; width++) {
            if(*bufpoint) {
                frame.pixel[width][height] = true;
            } else {
                frame.pixel[width][height] = false;
            }
            bufpoint++;
        }
    }
    m_count++;
    QString count = QString("Frames: %1").arg( m_count, 8, 10 );
    FrameCountLabel->setText(count);

    blinkSimWidget1->slotShowFrame(frame);
    
}

/*
 * public slot
 */
void VisionUDP::slotDnsReady()
{
    QValueList<QHostAddress> list = m_dns->addresses();
    QValueList<QHostAddress>::Iterator it = list.begin();
    if( it != list.end() ) {
        m_hostaddress = *it;
        PortLabel->setText(QString("IP: %1:%2")
                .arg(m_hostaddress.toString())
                .arg(m_settings->HostPort->value()));
        slotRefresh();
        m_timer->start(10000);
    } else {
        qDebug("Could not resolve hostname");
        close();
    }
    delete m_dns;
    m_dns = 0;
}

/*
 * public slot
 */
void VisionUDP::slotRefresh()
{

    char Buffer[11];
    // Magic
    Buffer[0] = (char)0xDE;
    Buffer[1] = (char)0xAA;
    Buffer[2] = (char)0xBF;
    Buffer[3] = (char)0xCD;
    Buffer[4] = 'R';
    Buffer[5] = 'E';
    Buffer[6] = 'F';
    Buffer[7] = 'R';
    Buffer[8] = 'E';
    Buffer[9] = 'S';
    Buffer[10] = 'H';

    if(!m_socket->writeBlock(Buffer, 11, m_hostaddress, 
                m_settings->HostPort->value()) == -1) {
        qDebug("VisionUDP: Could not write bytes");
    }
}
